//
//  BsWebViewController.swift
//  BaiSiSMApp
//
//  Created by davidhuang on 2022/11/11.
//  Copyright © 2022 www.davidhuang.com. All rights reserved.
//

import UIKit
import WebKit


extension WKWebView {
    open override var safeAreaInsets: UIEdgeInsets {
        return UIEdgeInsets(top: 0, left: 0, bottom: 0, right: 0)
    }
}

class YHWebViewController: YHBaseViewController  {
    public var isShowNavigation = true
    public var isPresent = false
    public var isShowShared = true
    // MARK: - 懒加载方法
    lazy var kvoHelper = BsKVOHelper()
    var urlString : String?
    var userAgent:String {
        get {
            if let content = WKWebView().value(forKey: "userAgent") as? String {
                return content
            } else {
                return ""
            }
        }
    }

    public func joinNav() -> UINavigationController {
        self.isPresent = true
        self.isShowShared = false
        let nav = YHNavigationController(rootViewController: self)
        nav.modalPresentationStyle = .fullScreen
        return nav
    }

    var isStatusBarHidden = false{
        didSet{
            if isStatusBarHidden != oldValue, isViewLoaded{
                setNeedsStatusBarAppearanceUpdate()
            }
        }
    }

    var statusBarStyle = UIStatusBarStyle.lightContent{
        didSet{
            if statusBarStyle != oldValue, isViewLoaded{
                setNeedsStatusBarAppearanceUpdate()
            }
        }
    }

    override var prefersStatusBarHidden: Bool{
        return isStatusBarHidden
    }

    override var preferredStatusBarStyle: UIStatusBarStyle{
        return statusBarStyle
    }

    var webView: WKWebView!

    lazy var progressView: UIProgressView = {
        let temp = UIProgressView()
        temp.tintColor = .mainColor
        temp.trackTintColor = .white
        return temp
    }()


    //是否加载了js方法
    var hasAddJsMethodFlag : Bool = false



    // MARK: - 生命周期方法
    deinit {
        if isViewLoaded {
            kvoHelper.remove(observer: self, atObject: webView, forKeyPath: "estimatedProgress")
            kvoHelper.remove(observer: self, atObject: webView, forKeyPath: "title")
        }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        setupSubview()
        addLayout()
        registeGlobalUserAgent()
        (self.navigationController as? YHNavigationController)?.myDelegate = self
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        if isShowNavigation == false {
            self.navigationController?.navigationBar.isHidden = true
        }

        if hasAddJsMethodFlag == false {
            webView.configuration.userContentController.add(self, name: "methodForJsInvoke")
            hasAddJsMethodFlag = true
        }
    }
    
    func deinitAction() {
        if isShowNavigation == false {
            self.navigationController?.navigationBar.isHidden = false
        }
        //不执行会导致循环引用
        webView?.configuration.userContentController.removeScriptMessageHandler(forName: "methodForJsInvoke")
        //webView.configuration.userContentController.removeAllContentRuleLists()
        (self.navigationController as? YHNavigationController)?.myDelegate = nil
    }
    
}

// MARK: - WKNavigationDelegate
extension YHWebViewController:WKUIDelegate,WKNavigationDelegate {

    func webView(_ webView: WKWebView, didFailProvisionalNavigation navigation: WKNavigation!, withError error: Error) {
        //        showEmptyView(showFlag: true)
    }

    func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        //        showEmptyView(showFlag: false)
    }

    func webView(_ webView: WKWebView, didStartProvisionalNavigation navigation: WKNavigation!) {
        //        showEmptyView(showFlag: false)
    }

    //    func webView(_ webView: WKWebView, didReceive challenge: URLAuthenticationChallenge, completionHandler: @escaping (URLSession.AuthChallengeDisposition, URLCredential?) -> Void) {
    ////        var host: String? = nil
    ////        if let ipHost = webView.url?.host,let realHost = hostMap[ipHost]{
    ////            host = realHost
    ////        }
    ////        sessionTaskDelegate.handlerChallenge(challenge, host: host,
    ////                                             completionHandler: completionHandler)
    //    }

    func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
        //        if let ipHost = navigationAction.request.url?.host,
        //           let realHost = hostMap[ipHost]{
        //            var copyRequest = (navigationAction.request as NSURLRequest).httpdns_getPostIncludeBody()
        //            if  copyRequest.allHTTPHeaderFields?["Host"] == nil{
        //                copyRequest.addValue(realHost, forHTTPHeaderField: "Host")
        //                webView.load(copyRequest)
        //                decisionHandler(.cancel)
        //                return
        //            }
        //        }
        //
        //        if let requestUrl = navigationAction.request.url?.absoluteString,
        //           requestUrl.hasPrefix(RyMTAHybRequest.prefix) {
        //            print("mta-ry: \(navigationAction.request.url?.absoluteString ?? "null")")
        //            var requestUrlRemovingPercentEncoding = requestUrl.removingPercentEncoding ?? requestUrl
        //            requestUrlRemovingPercentEncoding = requestUrlRemovingPercentEncoding.substring(from: RyMTAHybRequest.prefix.count)
        //            if let obj = RyMTAHybRequest(JSONString: requestUrlRemovingPercentEncoding), obj.trackEvent() {
        //                decisionHandler(.cancel)
        //                return
        //            }
        //            if MTAHybrid.handle(navigationAction, from: webView){
        //                decisionHandler(.cancel)
        //                return
        //            }
        //        }

        decisionHandler(.allow)
    }
}



extension YHWebViewController {


    func hideProgress() {
        progressView.isHidden = true
    }

    func showProgress()  {
        progressView.isHidden = false
    }

    func goBack(){
        if webView.window != nil,webView.canGoBack{
            webView.goBack()
        }else if navigationController?.viewControllers.count ?? 0 > 1{
            navigationController?.popViewController(animated: true)
            deinitAction()
        }else{
            dismiss(animated: true, completion: nil)
            deinitAction()
        }
    }



    func setupSubview(){
        view.backgroundColor = .white

        if let urlString = urlString {
            webView = BsWebViewPreloadManager.share.getWebView(urlString: urlString)
            webView.frame = UIScreen.main.bounds
            webView.scrollView.keyboardDismissMode = .onDrag
            webView.backgroundColor = .backgroundColor
            webView.isOpaque = false
            webView.uiDelegate = self
            webView.navigationDelegate = self
            webView.allowsLinkPreview = false
            webView.scrollView.bounces = false
            webView.scrollView.showsVerticalScrollIndicator = false
            webView.scrollView.showsHorizontalScrollIndicator = false
            kvoHelper.add(observer: self, toObject: webView, forKeyPath: "estimatedProgress", options: .new, context: nil)
            kvoHelper.add(observer: self, toObject: webView, forKeyPath: "title", options: .new, context: nil)
            view.addSubview(webView)
        } else {
            BsHUD.flash(message: "urlString 为空")
            //#if DEBUG
            //            if let url = Bundle.main.url(forResource: "WKWebView-WKScriptMessageHandler", withExtension: "html") {
            //                let request = URLRequest(url: url)
            //                webView = BsWebViewPreloadManager.share.onlyCreateWebView()
            //                webView.load(request)
            //                webView.navigationDelegate = self;
            //                view.addSubview(webView)
            //
            //                webView.scrollView.contentInsetAdjustmentBehavior = .never
            //
            //                let loginBtnItem = UIBarButtonItem(title: "测试", style: .plain, target: self, action: #selector(onClickTestItem))
            //                self.navigationItem.rightBarButtonItem = loginBtnItem
            //            }
            //#endif
        }
        
        view.addSubview(progressView)
        view.backgroundColor = .white
#if DEBUG
        let testBtn = UIButton(type: .custom)
        testBtn.setTitle("点我呀，豪哥", for: .normal)
        testBtn.setTitleColor(.mainColor, for: .normal)
        testBtn.backgroundColor = .purple
        view.addSubview(testBtn)

        testBtn.addTarget(self, action: #selector(onClickTestItem), for: .touchUpInside)

        testBtn.snp.makeConstraints { make in
            make.top.equalTo(200)
            make.right.equalTo(-16)
            make.height.equalTo(44)
            make.width.equalTo(150)
        }
        
        // TODO: - for test hjl 正式发布版本前需要屏蔽下面2个代码
        webView.backgroundColor = .purple
#endif

        if isPresent == true {
            self.bs_setupLeftButton(image: UIImage(named: "close_icon"))
        }
        
        if isShowShared == true {
            self.bs_setupRightButton(image: UIImage(named: "nav_share_icon"))
        }
    }


    // MARK: - event response 事件响应包括手势和按钮等
    //事件响应，包含手势、按钮、通知等等事件的处理
    @objc override func clickedNavRight() {

//        //1.判断是否登录
//        if BsLoginManager.checkLoginStatus(andLogin: true, invokeVc: self) == true {
//            BsHUD.flash(message: "分享功能还需要讨论")
//        }
    }


    @objc private func onClickTestItem() -> Void {

        let dic = ["type":"appToken","param":["token" : "豪哥你真帅~~~","key2":"value2"]] as [String : Any]
        let json = BSJsonUtil.dictionaryToJson(dic as NSDictionary)

        let jsString = "methodForAppInvoke('\(json)')"

        webView.evaluateJavaScript(jsString) { (result : Any?, error : Error?) in
            print("result")
            print(result as Any)

            print("error")
            print(error as Any)

        }
        self.webView.backgroundColor = .random
    }


    private func appPassTokenToH5() -> Void {
//        let dic = ["type":"appToken","param":["token" : BsLoginManager.shared.loginInfoModel?.access_token ?? ""]] as [String : Any]
//        let json = BSJsonUtil.dictionaryToJson(dic as NSDictionary)
//        let jsString = "methodForAppInvoke('\(json)')"
//        webView.evaluateJavaScript(jsString) { (result : Any?, error : Error?) in
//            print("result")
//            print(result as Any)
//
//            print("error")
//            print(error as Any)
//        }
    }

    func addLayout(){
        
        webView.snp.makeConstraints { (make) in
            if isShowNavigation == true {
                make.top.equalToSuperview().offset(UIDevice.bs_navigationFullHeight())
            } else {
                make.top.equalToSuperview()
            }
            make.left.equalToSuperview()
            make.right.equalToSuperview()
            make.bottom.equalToSuperview()
        }
        
        if isShowNavigation == true {
            progressView.snp.makeConstraints { (make) in
                make.top.equalTo(webView)
                make.left.equalToSuperview()
                make.right.equalToSuperview()
                make.height.equalTo(2)
            }
        }
        
    }

    private func loadURL(url : String? = nil){
        print("load url: \(url ?? "")")
        if let nsURL = URL(string: url){
            let request = URLRequest(url: nsURL)
            webView.load(request)
        }
    }


    func registeGlobalUserAgent() {
        assert(!self.userAgent.isEmpty)
        let newUserAgent = self.userAgent

        let version = UIDevice.appVersion() + "." + UIDevice.appBuild()

        /*
         env 环境： dev 开发、trial 体验、prod 正式
         token: 密钥
         immerse： true / false APP是否沉浸式
         scenes: app / mini / web 进入场景 (为后期做数据埋点上报使用)
         */


        let token = ""//BsLoginManager.shared.loginInfoModel?.access_token ?? ""
        let uid = ""//BsLoginManager.shared.loginInfoModel?.user_id ?? 0
        let phone_num = ""//BsLoginManager.shared.loginInfoModel?.phone_num ?? ""

        var dict = ["version":"\(version)",
                    "uid":"\(uid)",
                    "token":"\(token)",
                    "phone_num":"\(phone_num)",
                    "scenes":"iOS",
                    "system":newUserAgent,
                    "statusbar":["width":UIDevice.kScreenW,"height":UIDevice.bs_statusBarHeight()]
        ] as [String : Any]
#if DEBUG
        dict["appName"] = "优眠健康-Dev-iOS"
        dict["env"] = "dev"
#else
        dict["appName"] = "优眠健康-Release-iOS"
        dict["env"] = "prod"
#endif
        dict["sys-wkweb"] = newUserAgent
        let jsonString = dict.jsonString()
        webView.customUserAgent = jsonString
    }
}

// MARK: - KVO
extension YHWebViewController {
    override func observeValue(forKeyPath keyPath: String?, of object: Any?, change: [NSKeyValueChangeKey : Any]?, context: UnsafeMutableRawPointer?) {
        guard (object as? WKWebView) == webView else {
            super.observeValue(forKeyPath: keyPath, of: object, change: change, context: context)
            return
        }

        switch keyPath {
            case "estimatedProgress":
                let progress = Float(self.webView.estimatedProgress)
                progressView.setProgress(progress, animated: true)
                if progress >= 1.0 {
                    UIView.animate(withDuration: 0.3, delay: 0.1, options: .curveEaseOut, animations: {
                        self.progressView.alpha = 0
                    }, completion: { (finish) in
                        self.progressView.setProgress(0.0, animated: false)
                    })
                }else{
                    self.progressView.alpha = 1
                }
                break
            case "title":
                self.navigationItem.title = self.webView.title
                break
            default:
                break
        }
    }
}

extension YHWebViewController : WKScriptMessageHandler {
    func userContentController(_ userContentController: WKUserContentController, didReceive message: WKScriptMessage) {
        if message.name == "methodForJsInvoke" {
            print("message.name = \(message.name)")
            print("message.body = \(message.body)")

            /*
             var action  = {
             "type": "2", //业务类型
             "param": {"url":"https://www.baidu.com/","key2":"value2",.....}, //业务参数
             }
             */
            if message.body is [String : Any] == true {
                let dic = message.body as! [String : Any]
                if let type = dic["type"] as? String {

                    var param = dic["param"] as? Dictionary<String,Any>
                    if let temp = param {
                        param = temp
                    } else {
                        param = [:]
                    }


                    var title : String = type
                    switch type {
                        case "wxShare":
                            //微信分享
                            break
                        case "tabbar":
                            //tabbar 显示 隐藏
                            if  let show = param?["show"] as? String {
                                if show == "1" {
                                    NotificationCenter.default.post(name: BsConstant.BsNotification.tabBarShowNotification, object: nil)
                                } else {
                                    NotificationCenter.default.post(name: BsConstant.BsNotification.tabBarHideNotification, object: nil)
                                }
                            }
                            break
                        case "goMap":
                            //地图导航
                            break
                        case "appLogin":
                            //app登录
                            DispatchQueue.main.async {
//                                BsLoginManager.shared.showLoginVC(invokeVC: self) {
//                                    //登录成功后的回调
//                                    self.appPassTokenToH5()
//                                }
                            }
                            return
                        case "callPhone":
                            //拨打电话
                            if  let phone = param?["phone"] as? String {
                                phone.makePhone()
                            }
                            return
                        case "goBack":
                            self.goBack()
                            return
                        default:
                            print("未知的type 类型 \(type)")
                            break
                    }

                    var content : String = "无参数"
                    if let param = dic["param"] as? Dictionary<String,Any> {
                        content = BSJsonUtil.dictionaryToJson(param as NSDictionary)
                    }

                    title  = title + "\n 参数是:" + content
                    BsHUD.flash(message: title)
                }
            } else {
                print("h5返回数据可能有问题")
            }

        }
    }
}

// MARK: - BsNavigationControllerDelegate
extension YHWebViewController : BsNavigationControllerDelegate {
    func canBack() -> Bool {
        goBack()
        return false
    }
    
    override func callBackAction() {
        _ = canBack()
    }
}
